#include <Arduino.h>
#include <SoftwareSerial.h>
#include <EEPROM.h>
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266HTTPClient.h>
#include <Wire.h>
#include <SimpleDHT.h>
#include <Adafruit_BMP085.h>

#define CONNTIME  720000 // every 12 minutes (12 * 60 * 1000)

#define LENG 31   //0x42 + 31 bytes equal to 32 bytes
unsigned char buf[LENG];

#define LED_RED     0
#define LED_GREEN   14
#define LED_BLUE    12

String esid = "";
String epass = "";
bool wifiConnected = true;
bool dataSent = false;
unsigned long timeout = 0;

String data;
String pmdata;

SimpleDHT11 dht11;
Adafruit_BMP085 bmp;
SoftwareSerial PMSerial(13, 15); // RX, TX
WiFiServer server(80); //Server on port 80

byte pinDHT11 = 2;  // GPIO2 of ESP8266
int PM01Value=0;  //define PM1.0 value of the air detector module
int PM2_5Value=0;  //define PM2.5 value of the air detector module
int PM10Value=0;  //define PM10 value of the air detector module
long stationID = 0;
byte senddata = 0;

void ReadEEPROM() {
  Serial.println("Reading EEPROM ssid");
  esid = "";
  epass = "";
  for (int i = 0; i < 32; ++i) esid += char(EEPROM.read(i));
  for (int i = 32; i < 96; ++i) epass += char(EEPROM.read(i));
  esid = esid.substring(0,esid.indexOf(char(0)));
  epass = epass.substring(0,epass.indexOf(char(0)));
  Serial.println("SSID: " + esid);
  Serial.println("Passwd: " + epass);
}

void Write2EEPROM() {
  String qsid = esid;
  String qpass = epass;
  if (qsid.length() > 0 && qpass.length() > 0) {
    Serial.println("clearing eeprom");
    for (int i = 0; i < 96; ++i) { EEPROM.write(i, 0); }
    Serial.println(qsid);
    Serial.println("");
    Serial.println(qpass);
    Serial.println("");
      
    Serial.println("writing eeprom ssid:");
    for (int i = 0; i < qsid.length(); ++i)
      {
        EEPROM.write(i, qsid[i]);
        Serial.print("Wrote: ");
        Serial.println(qsid[i]); 
      }
    Serial.println("writing eeprom pass:"); 
    for (int i = 0; i < qpass.length(); ++i)
      {
        EEPROM.write(32+i, qpass[i]);
        Serial.print("Wrote: ");
        Serial.println(qpass[i]); 
      }    
    EEPROM.commit();
  } else Serial.println("Sending 404");
}

void rgbset(bool red, bool green, bool blue) {
  digitalWrite(LED_RED, !red);
  digitalWrite(LED_GREEN, !green);
  digitalWrite(LED_BLUE, !blue);
}

char checkValue(unsigned char *thebuf, char leng)
{  
  char receiveflag=0;
  int receiveSum=0;

  for(int i=0; i<(leng-2); i++){
  receiveSum=receiveSum+thebuf[i];
  }
  receiveSum=receiveSum + 0x42;
 
  if(receiveSum == ((thebuf[leng-2]<<8)+thebuf[leng-1]))  //check the serial data 
  {
    receiveSum = 0;
    receiveflag = 1;
  }
  return receiveflag;
}

int transmitPM01(unsigned char *thebuf)
{
  int PM01Val;
  PM01Val=((thebuf[3]<<8) + thebuf[4]); //count PM1.0 value of the air detector module
  return PM01Val;
}

//transmit PM Value to PC
int transmitPM2_5(unsigned char *thebuf) {
  int PM2_5Val;
  PM2_5Val=((thebuf[5]<<8) + thebuf[6]);//count PM2.5 value of the air detector module
  return PM2_5Val;
}

//transmit PM Value to PC
int transmitPM10(unsigned char *thebuf)
{
  int PM10Val;
  PM10Val=((thebuf[7]<<8) + thebuf[8]); //count PM10 value of the air detector module  
  return PM10Val;
}

void StartHotSpot(void) {
  const char* ssid = "LemATMO";
  const char* password = "12345678";
  WiFi.mode(WIFI_AP);

  IPAddress local_IP(192,168,4,22);
  IPAddress gateway(192,168,4,9);
  IPAddress subnet(255,255,255,0);
  Serial.print("\nSetting soft-AP configuration ... ");
  Serial.println(WiFi.softAPConfig(local_IP, gateway, subnet) ? "Ready" : "Failed!");
  Serial.print("Setting soft-AP ... ");
  Serial.println(WiFi.softAP(ssid, password) ? "Ready" : "Failed!");
 
  Serial.print("HotSpot IP: ");
  Serial.println(WiFi.softAPIP());

  ReadEEPROM();
  
  server.begin();                 
  Serial.println("HTTP server started");

  wifiConnected = false;
}
void HotSpot() {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) return;
  
  // Wait until the client sends some data
  Serial.println("\nWaiting for new client...\n");
  while(!client.available()) delay(1);
    
  // Read the first line of the request
  String request = client.readStringUntil('\r');
  Serial.println(request);
  if (request.indexOf("ssid") > -1 ) {
    String tmp = request.substring(request.indexOf("ssid"));
    tmp = tmp.substring(0,tmp.indexOf(" "));
    esid = tmp.substring(5,tmp.indexOf("&"));
    epass = tmp.substring(tmp.indexOf("&") + 6);
    Serial.println("SSID: " + esid); 
    Serial.println("Passwd: " + epass);
    client.println("HTTP/1.1 200 OK");
    client.println("Content-Type: text/html");
    client.println(""); //  do not forget this one
    client.println("Zmieniono ustawienia. LemATMO zostanie zrestartowane.");
    delay(1000);
    Write2EEPROM();
    Serial.println("\nRebooting device.");
    ESP.restart();
  } else if (request.indexOf("/getwifi") == -1 ) {
   ReadEEPROM();
  }
  
  client.flush();

  // Scan for networks
  Serial.println("\nScan: start");
  String WiFinetworks = "";
  int n = WiFi.scanNetworks();
  Serial.println("Scan: done");
  if (request.indexOf("/getwifi") == -1 ) {
    if (n == 0) {
      Serial.println("no networks found");
    } else {
      Serial.print(n);
      Serial.println(" networks found");
      for (int i = 0; i < n; ++i) {
        // Print SSID and RSSI for each network found
        Serial.print(i + 1);
        Serial.print(": ");
        Serial.print(WiFi.SSID(i));
        WiFinetworks += "<option value=\"" + WiFi.SSID(i) + "\" " + ( WiFi.SSID(i) == esid ? "selected " : "") + "/>" + WiFi.SSID(i) + "</option>";
        Serial.print(" (");
        Serial.print(WiFi.RSSI(i));
        Serial.print(")");
        Serial.println((WiFi.encryptionType(i) == ENC_TYPE_NONE) ? " " : "*");
        delay(10);
      }
      client.println("HTTP/1.1 200 OK");
      client.println("Content-Type: text/html");
      client.println(""); //  do not forget this one
      client.println("<!DOCTYPE HTML>");
      client.println("<html>");
      client.println("<head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" /><title>LemATMO Setup</title></head>");
      client.println("<body>");
      client.println("<style>");
      client.println("body {align: center; vertical-align: middle; text-align: center; width: 100%; background-image: -moz-radial-gradient(#cde6f9, #a1c8eb); background-image: -webkit-gradient(radial, 50% 50%, 0, 50% 50%,400, from(#cde6f9), to(#a1c8eb)); font-size: 3em; font-family: 'Helvetica', 'Arial', sans-serif; }");
      client.println("h2 {font-size: 2em; color: #fff; text-shadow: 2px 2px 10px rgba(0,0,0,.6);}");
      client.println("label {width: 40%; text-align: right;}");
      client.println("select, input { width: 240px; text-align: left; font-size: 1em; margin: 0 2%; padding: .2em .4em; text-align: center;}");
      client.println("input#button {font-size: 1.2em; width: 160px; height: 20px; margin: 4px; /*padding: .4em 0 .4em*/; text-align: center; box-shadow: 0 0 20px rgba(255,255,255,.6); min-height: 2em;}");
      client.println(" @media only screen and (max-width:320px) { body {font-size: 12px} }");
      client.println(" @media only screen and (max-width:360px) { body {font-size: 14px} }");
      client.println(".clr { clear: both: margin: .2em 0; float: none; }");
      client.println("</style>");
      client.print("<h2>LemATMO Setup</h2> ");
      client.println("<br/><br/><form method=\"get\"");
      client.println("<label for=\"ssid\">Sieć: </label><br/><select name=\"ssid\">");
      client.println(WiFinetworks);
      client.println("</select><br style=\"padding: 2%\"/><div class=\"clr\"></div><br/>");
      client.println("<label for=\"pass\">Hasło: </label><br/><input type=\"text\" name=\"pass\" value=\"" + epass + "\"><div class=\"clr\"></div><br/><br/>");
      client.println("<input type=\"submit\" value=\"Ustaw\" id=\"button\"/>");
      client.println("</form></body></html>");
    }
  } else {
    Serial.print(n);
      Serial.println(" networks found");
      for (int i = 0; i < n; ++i) {
        // Print SSID and RSSI for each network found
        Serial.print(i + 1);
        Serial.print(": ");
        Serial.print(WiFi.SSID(i));
        WiFinetworks += WiFi.SSID(i) + ",";
        Serial.print(" (");
        Serial.print(WiFi.RSSI(i));
        Serial.print(")");
        Serial.println((WiFi.encryptionType(i) == ENC_TYPE_NONE) ? " " : "*");
        delay(10);
      }
      client.println("HTTP/1.1 200 OK");
      client.println("Content-Type: text/html");
      client.println(""); //  do not forget this one
      client.println(WiFinetworks.substring(0, WiFinetworks.length() - 1));
  }
  
  delay(1);
  Serial.println("Client disconnected");
  Serial.println(""); 
}

void WiFiConnect() {
  timeout = 0;
  if (epass != "" && esid != "") {
    Serial.print("Connecting to ");
    Serial.println(esid);
    
    WiFi.begin(esid.c_str(),epass.c_str());

    byte count = 0;
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
      count++;
      if (count > 20) {
        wifiConnected = false;
        break;
      }
    }
    
    if (wifiConnected) {
      Serial.println("\nWiFi connected.");
      WiFi.mode(WIFI_STA);
      //rgbset(0,1,0);
    } else StartHotSpot();
  } else StartHotSpot();
}

void setup(void){
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);

  digitalWrite(LED_RED, LOW);
  digitalWrite(LED_GREEN, LOW);
  digitalWrite(LED_BLUE, LOW);
  
  stationID = ESP.getChipId();
  
  Serial.begin(115200);
  while (!Serial) { }

  PMSerial.begin(9600);   
  PMSerial.setTimeout(1500);

  EEPROM.begin(512);
  delay(10);
  Serial.println();
  Serial.println();
  Serial.println("Startup");
  Serial.println("Chip ID: " + String(stationID));

  ReadEEPROM();
  WiFiConnect();
  
}

void loop(void){
   if (epass != "" && esid != "") {
    if (timeout >= 6e+7) WiFiConnect();
    else timeout++;
   }
   if (wifiConnected) {
      // ------------------- AIR QUALITY -------------------
      byte i = 0;
      do {
        i++;
        Serial.println("\nReading sensors:\n");
        if(PMSerial.find(0x42)){  
          Serial.println("\nSample air quality...");
          PMSerial.readBytes(buf,LENG);
      
          if(buf[0] == 0x4d){
            if(checkValue(buf,LENG)){
              PM01Value = transmitPM01(buf); //count PM1.0 value of the air detector module
              PM2_5Value = transmitPM2_5(buf);//count PM2.5 value of the air detector module
              PM10Value = transmitPM10(buf); //count PM10 value of the air detector module 
            } else return;
          } else return;
        }
          
        delay(500);
      } while ( PM10Value == 0 && i < 100 );

      Serial.print("PM1.0: ");  
      Serial.print(PM01Value);
      Serial.println("  ug/m3");            
      
      Serial.print("PM2.5: ");  
      Serial.print(PM2_5Value);
      Serial.println("  ug/m3");     
      
      Serial.print("PM10: ");  
      Serial.print(PM10Value);
      Serial.println("  ug/m3");
      
      if (PM10Value < 50) rgbset(0,1,0);
      else if (PM10Value < 100) rgbset(1,1,0);
      else rgbset(1,0,0);
      
    // ------------------- TEMPERATURE & HUMIDITY -------------------
    Serial.println("\nSample DHT11..."); 
     
    byte temperature = 0;
    byte humidity = 0;
    byte count = 0;
    int err = SimpleDHTErrSuccess;
    while ((err = dht11.read(pinDHT11, &temperature, &humidity, NULL)) != SimpleDHTErrSuccess || count > 4) {
      delay(500);
      count++;
      if (count > 5) {
        Serial.println("Read DHT11 failed, err=" + String(err));
        break;
      }
    }
    Serial.print("Sample OK: ");
    Serial.print((int)temperature);
    Serial.print(" *C, "); 
    Serial.print((int)humidity);
    Serial.println(" %");
      
    // ------------------- TEMPERATURE & PRESSURE -------------------
      if (!bmp.begin()) {
        Serial.println("\nCould not find a valid BMP180 sensor, check wiring!");
      } else {
        Serial.println("\nReading BMP180 sensor...");
        byte bmpTEMP = bmp.readTemperature();
        int bmpPRESSURE = bmp.readPressure() / 100;
        Serial.print("Temperature = ");
        Serial.print(bmpTEMP);
        Serial.println(" *C");
        
        Serial.print("Pressure = ");
        Serial.print(bmpPRESSURE);
        Serial.println(" hPa\n\n");
      }
      
  
    if (WiFi.status() == WL_CONNECTED) {
      PM01Value = 0;
      PM2_5Value = 0;
      PM10Value = 0;
      delay(5000);
    } else WiFiConnect();
    
   } else {
    rgbset(1,0,1);
    HotSpot();
   }
}


